@@ -1,6 +1,9 @@
+from __future__ import unicode_literals
+
 import copy
-import re
-import datetime
+
+from six import string_types
+from six import text_type
 
 from taggit.forms import TagWidget
 from modelcluster.forms import ClusterForm, ClusterFormMetaclass
@@ -9,147 +12,23 @@ from django.template.loader import render_to_string
 from django.template.defaultfilters import addslashes
 from django.utils.safestring import mark_safe
 from django import forms
-from django.db import models
 from django.forms.models import fields_for_model
 from django.contrib.contenttypes.models import ContentType
-from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured, ValidationError
+from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured
 from django.core.urlresolvers import reverse
-from django.conf import settings
-from django.utils.translation import ugettext as _
 from django.utils.translation import ugettext_lazy
 
 from wagtail.wagtailcore.models import Page
-from wagtail.wagtailcore.util import camelcase_to_underscore
+from wagtail.wagtailcore.utils import camelcase_to_underscore
 from wagtail.wagtailcore.fields import RichTextArea
 
 
-class FriendlyDateInput(forms.DateInput):
-    """
-    A custom DateInput widget that formats dates as "05 Oct 2013"
-    and adds class="friendly_date" to be picked up by jquery datepicker.
-    """
-    def __init__(self, attrs=None):
-        default_attrs = {'class': 'friendly_date'}
-        if attrs:
-            default_attrs.update(attrs)
-
-        super(FriendlyDateInput, self).__init__(attrs=default_attrs, format='%d %b %Y')
-
-
-class FriendlyTimeInput(forms.TimeInput):
-    """
-    A custom TimeInput widget that formats dates as "5.30pm"
-    and adds class="friendly_time" to be picked up by jquery timepicker.
-    """
-    def __init__(self, attrs=None):
-        default_attrs = {'class': 'friendly_time'}
-        if attrs:
-            default_attrs.update(attrs)
-
-        super(FriendlyTimeInput, self).__init__(attrs=default_attrs, format='%I.%M%p')
-
-
-class FriendlyTimeField(forms.CharField):
-    def to_python(self, time_string):
-        # Check if the string is blank
-        if not time_string:
-            return None
-
-        # Look for time in the string
-        expr = re.compile("^(?P<hour>\d+)(?:(?:.|:)(?P<minute>\d+))?(?P<am_pm>am|pm)")
-        match = expr.match(time_string.lower())
-        if match:
-            # Pull out values from string
-            hour_string, minute_string, am_pm = match.groups()
-
-            # Convert hours and minutes to integers
-            hour = int(hour_string)
-            if minute_string:
-                minute = int(minute_string)
-            else:
-                minute = 0
-
-            # Create python time
-            if am_pm == "pm" and hour < 12:
-                hour += 12
-
-            if am_pm == "am" and hour >= 12:
-                hour -= 12
-
-            return datetime.time(hour=hour, minute=minute)
-        else:
-            raise ValidationError(_("Please type a valid time"))
-
-
-class LocalizedDateInput(forms.DateInput):
-    """
-    A custom DateInput widget that formats localized dates
-    and adds class="friendly_date" to be picked up by jquery datepicker.
-    """
-    def __init__(self, attrs=None):
-        default_attrs = {'class': 'localized_date', 'localize':True}
-        if attrs:
-            default_attrs.update(attrs)
-
-        super(LocalizedDateInput, self).__init__(attrs=default_attrs)
-
-
-class LocalizedTimeInput(forms.TimeInput):
-    """
-    A custom TimeInput widget that formats dates as "5.30pm"
-    and adds class="friendly_time" to be picked up by jquery timepicker.
-    """
-    def __init__(self, attrs=None):
-        default_attrs = {'class': 'localized_time'}
-        if attrs:
-            default_attrs.update(attrs)
-        # Just use 24-hour format
-        super(LocalizedTimeInput, self).__init__(attrs=default_attrs, format='%H:%M')
-
-
-class LocalizedTimeField(forms.CharField):
-    def to_python(self, time_string):
-        # Check if the string is blank
-        if not time_string:
-            return None
-
-        # Look for time in the string
-        expr = re.compile("^(?P<hour>\d+)(?:(?:.|:)(?P<minute>\d+))?")
-        match = expr.match(time_string.lower())
-        if match:
-            # Pull out values from string
-            hour_string, minute_string= match.groups()
-
-            # Convert hours and minutes to integers
-            hour = int(hour_string)
-            if minute_string:
-                minute = int(minute_string)
-            else:
-                minute = 0
-            if hour>=24 or hour < 0 or minute >=60 or minute < 0:
-                raise ValidationError(_("Please type a valid time"))
-
-            return datetime.time(hour=hour, minute=minute)
-        else:
-            raise ValidationError(_("Please type a valid time") )
-
-
-if hasattr(settings, 'USE_L10N') and settings.USE_L10N==True:
-    FORM_FIELD_OVERRIDES = {
-        models.DateField: {'widget': LocalizedDateInput},
-        models.TimeField: {'widget': LocalizedTimeInput, 'form_class': LocalizedTimeField},
-    }
-else: # Fall back to friendly date/time
-    FORM_FIELD_OVERRIDES = {
-        models.DateField: {'widget': FriendlyDateInput},
-        models.TimeField: {'widget': FriendlyTimeInput, 'form_class': FriendlyTimeField},
-    }
+FORM_FIELD_OVERRIDES = {}
 
 WIDGET_JS = {
-    FriendlyDateInput: (lambda id: "initFriendlyDateChooser(fixPrefix('%s'));" % id),
-    FriendlyTimeInput: (lambda id: "initFriendlyTimeChooser(fixPrefix('%s'));" % id),
-    LocalizedDateInput: (lambda id: "initLocalizedDateChooser(fixPrefix('%s'));" % id),
-    LocalizedTimeInput: (lambda id: "initLocalizedTimeChooser(fixPrefix('%s'));" % id),
+    forms.DateInput: (lambda id: "initDateChooser(fixPrefix('%s'));" % id),
+    forms.TimeInput: (lambda id: "initTimeChooser(fixPrefix('%s'));" % id),
+    forms.DateTimeInput: (lambda id: "initDateTimeChooser(fixPrefix('%s'));" % id),
     RichTextArea: (lambda id: "makeRichTextEditable(fixPrefix('%s'));" % id),
     TagWidget: (
         lambda id: "initTagField(fixPrefix('%s'), '%s');" % (
@@ -193,7 +72,7 @@ class WagtailAdminModelFormMetaclass(ClusterFormMetaclass):
         new_class = super(WagtailAdminModelFormMetaclass, cls).__new__(cls, name, bases, attrs)
         return new_class
 
-WagtailAdminModelForm = WagtailAdminModelFormMetaclass('WagtailAdminModelForm', (ClusterForm,), {})
+WagtailAdminModelForm = WagtailAdminModelFormMetaclass(str('WagtailAdminModelForm'), (ClusterForm,), {})
 
 # Now, any model forms built off WagtailAdminModelForm instead of ModelForm should pick up
 # the nice form fields defined in FORM_FIELD_OVERRIDES.
@@ -229,7 +108,7 @@ def get_form_for_model(
     # Give this new form class a reasonable name.
     class_name = model.__name__ + str('Form')
     form_class_attrs = {
-        'Meta': type('Meta', (object,), attrs)
+        'Meta': type(str('Meta'), (object,), attrs)
     }
 
     return WagtailAdminModelFormMetaclass(class_name, (WagtailAdminModelForm,), form_class_attrs)
@@ -261,6 +140,10 @@ def extract_panel_definitions_from_model_class(model, exclude=None):
     return panels
 
 
+def set_page_edit_handler(page_class, handlers):
+    page_class.handlers = handlers
+
+
 class EditHandler(object):
     """
     Abstract class providing sensible default behaviours for objects implementing
@@ -304,19 +187,21 @@ class EditHandler(object):
     heading = ""
     help_text = ""
 
-    def object_classnames(self):
+    def classes(self):
         """
-        Additional classnames to add to the <li class="object"> when rendering this
-        within an ObjectList
+        Additional CSS classnames to add to whatever kind of object this is at output. 
+        Subclasses of EditHandler should override this, invoking super(B, self).classes() to
+        append more classes specific to the situation.
         """
-        return ""
 
-    def field_classnames(self):
-        """
-        Additional classnames to add to the <li> when rendering this within a
-        <ul class="fields">
-        """
-        return ""
+        classes = []
+
+        try:
+            classes.append(self.classname)
+        except AttributeError:
+            pass
+
+        return classes
 
     def field_type(self):
         """
@@ -360,12 +245,12 @@ class EditHandler(object):
         """
         rendered_fields = self.rendered_fields()
         missing_fields_html = [
-            unicode(self.form[field_name])
+            text_type(self.form[field_name])
             for field_name in self.form.fields
             if field_name not in rendered_fields
         ]
 
-        return mark_safe(u''.join(missing_fields_html))
+        return mark_safe(''.join(missing_fields_html))
 
     def render_form_content(self):
         """
@@ -382,12 +267,6 @@ class BaseCompositeEditHandler(EditHandler):
     """
     _widget_overrides = None
 
-    def object_classnames(self):
-        try:
-            return "multi-field " + self.classname
-        except (AttributeError, TypeError):
-            return "multi-field"
-
     @classmethod
     def widget_overrides(cls):
         if cls._widget_overrides is None:
@@ -425,7 +304,7 @@ class BaseCompositeEditHandler(EditHandler):
         }))
 
     def render_js(self):
-        return mark_safe(u'\n'.join([handler.render_js() for handler in self.children]))
+        return mark_safe('\n'.join([handler.render_js() for handler in self.children]))
 
     def rendered_fields(self):
         result = []
@@ -440,26 +319,41 @@ class BaseTabbedInterface(BaseCompositeEditHandler):
 
 
 def TabbedInterface(children):
-    return type('_TabbedInterface', (BaseTabbedInterface,), {'children': children})
+    return type(str('_TabbedInterface'), (BaseTabbedInterface,), {'children': children})
 
 
 class BaseObjectList(BaseCompositeEditHandler):
     template = "wagtailadmin/edit_handlers/object_list.html"
 
 
-def ObjectList(children, heading=""):
-    return type('_ObjectList', (BaseObjectList,), {
+def ObjectList(children, heading="", classname=""):
+    return type(str('_ObjectList'), (BaseObjectList,), {
         'children': children,
         'heading': heading,
+        'classname': classname
     })
 
 
+class BaseFieldRowPanel(BaseCompositeEditHandler):
+    template = "wagtailadmin/edit_handlers/field_row_panel.html"
+
+def FieldRowPanel(children, classname=""):
+    return type(str('_FieldRowPanel'), (BaseFieldRowPanel,), {
+        'children': children,
+        'classname': classname,
+    })
+
 class BaseMultiFieldPanel(BaseCompositeEditHandler):
     template = "wagtailadmin/edit_handlers/multi_field_panel.html"
 
+    def classes(self):
+        classes = super(BaseMultiFieldPanel, self).classes()
+        classes.append("multi-field")
+   
+        return classes
 
-def MultiFieldPanel(children, heading="", classname=None):
-    return type('_MultiFieldPanel', (BaseMultiFieldPanel,), {
+def MultiFieldPanel(children, heading="", classname=""):
+    return type(str('_MultiFieldPanel'), (BaseMultiFieldPanel,), {
         'children': children,
         'heading': heading,
         'classname': classname,
@@ -474,25 +368,23 @@ class BaseFieldPanel(EditHandler):
         self.heading = self.bound_field.label
         self.help_text = self.bound_field.help_text
 
-    def object_classnames(self):
-        try:
-            return "single-field " + self.classname
-        except (AttributeError, TypeError):
-            return "single-field"
+    def classes(self):
+        classes = super(BaseFieldPanel, self).classes();
+
+        if self.bound_field.field.required:
+            classes.append("required")
+        if self.bound_field.errors:
+            classes.append("error")
+        
+        classes.append(self.field_type())
+        classes.append("single-field")
+        
+        return classes
 
     def field_type(self):
         return camelcase_to_underscore(self.bound_field.field.__class__.__name__)
 
-    def field_classnames(self):
-        classname = self.field_type()
-        if self.bound_field.field.required:
-            classname += " required"
-        if self.bound_field.errors:
-            classname += " error"
-
-        return classname
-
-    object_template = "wagtailadmin/edit_handlers/field_panel_object.html"
+    object_template = "wagtailadmin/edit_handlers/single_field_panel.html"
 
     def render_as_object(self):
         return mark_safe(render_to_string(self.object_template, {
@@ -522,8 +414,8 @@ class BaseFieldPanel(EditHandler):
         return [self.field_name]
 
 
-def FieldPanel(field_name, classname=None):
-    return type('_FieldPanel', (BaseFieldPanel,), {
+def FieldPanel(field_name, classname=""):
+    return type(str('_FieldPanel'), (BaseFieldPanel,), {
         'field_name': field_name,
         'classname': classname,
     })
@@ -535,7 +427,7 @@ class BaseRichTextFieldPanel(BaseFieldPanel):
 
 
 def RichTextFieldPanel(field_name):
-    return type('_RichTextFieldPanel', (BaseRichTextFieldPanel,), {
+    return type(str('_RichTextFieldPanel'), (BaseRichTextFieldPanel,), {
         'field_name': field_name,
     })
 
@@ -591,7 +483,7 @@ class BasePageChooserPanel(BaseChooserPanel):
     def target_content_type(cls):
         if cls._target_content_type is None:
             if cls.page_type:
-                if isinstance(cls.page_type, basestring):
+                if isinstance(cls.page_type, string_types):
                     # translate the passed model name into an actual model class
                     from django.db.models import get_model
                     try:
@@ -626,7 +518,7 @@ class BasePageChooserPanel(BaseChooserPanel):
 
 
 def PageChooserPanel(field_name, page_type=None):
-    return type('_PageChooserPanel', (BasePageChooserPanel,), {
+    return type(str('_PageChooserPanel'), (BasePageChooserPanel,), {
         'field_name': field_name,
         'page_type': page_type,
     })
@@ -683,6 +575,11 @@ class BaseInlinePanel(EditHandler):
                 child_edit_handler_class(instance=subform.instance, form=subform)
             )
 
+        # if this formset is valid, it may have been re-ordered; respect that
+        # in case the parent form errored and we need to re-render
+        if self.formset.can_order and self.formset.is_valid():
+            self.children = sorted(self.children, key=lambda x: x.form.cleaned_data['ORDER'])
+
         empty_form = self.formset.empty_form
         empty_form.fields['DELETE'].widget = forms.HiddenInput()
         if self.formset.can_order:
@@ -709,7 +606,7 @@ class BaseInlinePanel(EditHandler):
 
 def InlinePanel(base_model, relation_name, panels=None, label='', help_text=''):
     rel = getattr(base_model, relation_name).related
-    return type('_InlinePanel', (BaseInlinePanel,), {
+    return type(str('_InlinePanel'), (BaseInlinePanel,), {
         'relation_name': relation_name,
         'related': rel,
         'panels': panels,
@@ -718,10 +615,23 @@ def InlinePanel(base_model, relation_name, panels=None, label='', help_text=''):
     })
 
 
+# This allows users to include the publishing panel in their own per-model override
+# without having to write these fields out by hand, potentially losing 'classname' 
+# and therefore the associated styling of the publishing panel
+def PublishingPanel():
+    return MultiFieldPanel([
+        FieldRowPanel([
+            FieldPanel('go_live_at'),
+            FieldPanel('expire_at'),
+        ], classname="label-above"),   
+    ], ugettext_lazy('Scheduled publishing'), classname="publishing")
+
+
 # Now that we've defined EditHandlers, we can set up wagtailcore.Page to have some.
 Page.content_panels = [
     FieldPanel('title', classname="full title"),
 ]
+
 Page.promote_panels = [
     MultiFieldPanel([
         FieldPanel('slug'),
@@ -730,3 +640,7 @@ Page.promote_panels = [
         FieldPanel('search_description'),
     ], ugettext_lazy('Common page configuration')),
 ]
+
+Page.settings_panels = [
+    PublishingPanel()
+]
