@@ -1,17 +1,20 @@
+import datetime
+
 from django.db import models
 from django.utils import timezone
+from django.utils.encoding import python_2_unicode_compatible
 
-from indexed import Indexed
-import datetime
-import string
+from wagtail.wagtailsearch import indexed
+from wagtail.wagtailsearch.utils import normalise_query_string, MAX_QUERY_STRING_LENGTH
 
 
+@python_2_unicode_compatible
 class Query(models.Model):
-    query_string = models.CharField(max_length=255, unique=True)
+    query_string = models.CharField(max_length=MAX_QUERY_STRING_LENGTH, unique=True)
 
     def save(self, *args, **kwargs):
         # Normalise query string
-        self.query_string = self.normalise_query_string(self.query_string)
+        self.query_string = normalise_query_string(self.query_string)
 
         super(Query, self).save(*args, **kwargs)
 
@@ -22,7 +25,7 @@ class Query(models.Model):
         daily_hits.hits = models.F('hits') + 1
         daily_hits.save()
 
-    def __unicode__(self):
+    def __str__(self):
         return self.query_string
 
     @property
@@ -39,26 +42,13 @@ class Query(models.Model):
 
     @classmethod
     def get(cls, query_string):
-        return cls.objects.get_or_create(query_string=cls.normalise_query_string(query_string))[0]
+        return cls.objects.get_or_create(query_string=normalise_query_string(query_string))[0]
 
     @classmethod
     def get_most_popular(cls, date_since=None):
         # TODO: Implement date_since
         return cls.objects.filter(daily_hits__isnull=False).annotate(_hits=models.Sum('daily_hits__hits')).distinct().order_by('-_hits')
 
-    @staticmethod
-    def normalise_query_string(query_string):
-        # Convert query_string to lowercase
-        query_string = query_string.lower()
-
-        # Strip punctuation characters
-        query_string = ''.join([c for c in query_string if c not in string.punctuation])
-
-        # Remove double spaces
-        query_string = ' '.join(query_string.split())
-
-        return query_string
-
 
 class QueryDailyHits(models.Model):
     query = models.ForeignKey(Query, db_index=True, related_name='daily_hits')
@@ -86,24 +76,8 @@ class EditorsPick(models.Model):
     sort_order = models.IntegerField(null=True, blank=True, editable=False)
     description = models.TextField(blank=True)
 
+    def __repr__(self):
+        return 'EditorsPick(query="' + self.query.query_string + '", page="' + self.page.title + '")'
+
     class Meta:
         ordering = ('sort_order', )
-
-
-# Used for tests
-
-class SearchTest(models.Model, Indexed):
-    title = models.CharField(max_length=255)
-    content = models.TextField()
-    live = models.BooleanField(default=False)
-
-    indexed_fields = ("title", "content", "callable_indexed_field", "live")
-
-    def callable_indexed_field(self):
-        return "Callable"
-
-
-class SearchTestChild(SearchTest):
-    extra_content = models.TextField()
-
-    indexed_fields = "extra_content"
